package de.intevation.flys.artifacts;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import org.w3c.dom.Document;

import de.intevation.artifactdatabase.state.DefaultFacet;
import de.intevation.artifactdatabase.state.Facet;
import de.intevation.artifactdatabase.state.DefaultOutput;
import de.intevation.artifactdatabase.state.State;

import de.intevation.artifacts.Artifact;
import de.intevation.artifacts.ArtifactFactory;
import de.intevation.artifacts.CallContext;
import de.intevation.artifacts.CallMeta;
import de.intevation.flys.artifacts.model.RiverFactory;

import de.intevation.flys.artifacts.model.FacetTypes;
import de.intevation.flys.artifacts.states.StaticState;

import de.intevation.flys.model.Gauge;
import de.intevation.flys.model.River;


/**
 * Artifact to access names of Points Of Interest along a segment of a river.
 * This artifact neglects (Static)FLYSArtifacts capabilities of interaction
 * with the StateEngine by overriding the getState*-methods.
 */
public class MainValuesArtifact
extends      StaticFLYSArtifact
{
    /** The logger for this class. */
    private static Logger logger = Logger.getLogger(MainValuesArtifact.class);

    /** The name of the artifact. */
    public static final String ARTIFACT_NAME = "mainvalue";

    /** One and only state to be in. */
    protected transient State state = null;


    /**
     * Trivial Constructor.
     */
    public MainValuesArtifact() {
        logger.debug("MainValuesArtifact.MainValuesartifact()");
    }


    /**
     * Gets called from factory, to set things up.
     */
    @Override
    public void setup(
        String          identifier,
        ArtifactFactory factory,
        Object          context,
        CallMeta        callMeta,
        Document        data)
    {
        logger.debug("MainValuesArtifact.setup");
        super.setup(identifier, factory, context, callMeta, data);
    }


    /**
     * Get a list containing the one and only State.
     * @param context ignored.
     * @return list with one and only state.
     */
    @Override
    protected List<State> getStates(Object context) {
        ArrayList<State> states = new ArrayList<State>();
        states.add(getState(null,null));
        return states;
    }


    /**
     * Get the "current" state.
     * @param cc ignored.
     * @return the "current" state.
     */
    @Override
    protected State getCurrentState(Object cc) {
        return getState();
    }


    /**
     * Get the only possible state.
     * @return the state.
     */
    protected State getState() {
        return getState(null, null);
    }


    /**
     * Get the state.
     * @param context ignored.
     * @param stateID ignored.
     * @return the state.
     */
    @Override
    protected State getState(Object context, String stateID) {
        if (state != null) {
            
        }
        else {
            state = new StaticState();
            List<Facet> fs = new ArrayList<Facet>();
            fs.add(new MainValuesFacet());
            // TODO check if facets and outputs already exist.
            // TODO also check, this is usually done in initialize, too.
            facets.put(state.getID(), fs);
            DefaultOutput mainValuesOutput1 = new DefaultOutput(
                        "discharge_curve", "output.discharge_curve", "image/png",
                        fs,
                        "chart");
            DefaultOutput mainValuesOutput2 = new DefaultOutput(
                        "computed_discharge_curve",
                        "output.computed_discharge_curve", "image/png",
                        fs,
                        "chart");
    
            state.getOutputs().add(mainValuesOutput1);
            state.getOutputs().add(mainValuesOutput2);
        }
        return state;
    }


        state.getOutputs().add(mainValuesOutput);
        return state;
    }


    /* FACET IMPLEMENTATION  */
    class MainValuesFacet extends DefaultFacet {

        public MainValuesFacet()
        {
        description = "facet.mainvalues";
        name = "facet.mainvalues";
        index = 0;
        }

        // TODO implement; what is index used for?
        /**
         * Returns the index of this facet.
         *
         * @return the index of this facet.
         */
        public int getIndex() {
            return 0;
        }
 
 
        /**
         * Returns the name of this facet.
         *
         * @return the name of this facet.
         */
        public String getName() {
            // TODO define, static
            // TODO remove, is part of DefaultFacet.
            return "facet.mainvalue";
        }
 
 
        /**
         * Returns the description of this facet.
         *
         * @return the description of this facet.
         */
        public String getDescription() {
            // TODO remove, is part of DefaultFacet.
            return "facet.mainvalues";
        }
 
 
        /**
         * Returns the data this facet requires.
         *
         * @param artifact The owner artifact.
         * @param context The CallContext.
         *
         * @return the data.
         */
        public Object getData(Artifact artifact, CallContext context) {
            return null;
        }
 

        /**
         * Create a deep copy of this Facet.
         * @return a deep copy.
         */ 
        public MainValuesFacet deepCopy() {
            MainValuesFacet copy = new MainValuesFacet();
            copy.set(this);
            return copy;
        }
    }
 
}
